/* To compile: f77 -o sample sample.c -lprimme -llapack -lgoto */

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "primme.h"

/* matrix vector multiplications */
static void
matveca(void *x0, void *y0, int *blockSize, struct primme_params *primme)
{
  double *x = x0, *y = y0;
  int i, n = primme->n;

  for (i = 0; i < n; i++)
	*(y++) = *(x++) * (i + 1.0);
}

static void
matvecb(void *x0, void *y0, int *blockSize, struct primme_params *primme)
{
    double *x = x0, *y = y0;
    memcpy(y, x, primme->n * sizeof(*x));
}

/* diagonal preconditioner */
static void
prec_diag(void *x0, void *y0, int *blockSize, struct primme_params *primme)
{
    double *x = x0, *y = y0;
    int i, n = primme->n;

    for (i = 0; i < n; i ++)
	y[i] = x[i] / (i + 1.0 - 0.0);
}

int
main(int argc, char **argv)
{
  int n = 100;
  int nevals = 5;
  double *evals = malloc(nevals * sizeof(*evals));
  double *evecs = malloc(nevals * n * sizeof(*evecs));
  double *rnorms = malloc(nevals * sizeof(*rnorms));
  double tau = 0.0;
  int i, ret;
  primme_params primme;

  primme_initialize(&primme);
  primme.n = n;	
  primme.numEvals = nevals;
  primme.matrixMatvec = matveca;
  primme.massMatrixMatvec = matvecb;
  primme.applyPreconditioner = prec_diag;

  /* primme.basisSize = ;*/
  /* primme.blockSize = ;*/

#if 0
  /* parallel PRIMME */
  primme.nLocal = n;
  primme.globalSumDouble =
	void globalSumDouble(double *sendBuf, double *recvBuf,
				int *count, primme_params *primme);
#endif

  primme_set_method(DYNAMIC, &primme);

  ret = dprimme(evals, evecs, rnorms, &primme);
  fprintf(stderr, "ret = %d\n", ret);

  for (i = 0; i < nevals; i++) {
    fprintf(stderr, "eigenvalue %d: %0.16lg\n", i + 1, evals[i]);
  }
  
  return 0;
}
